-------------------------------------------------------------------------------
--
-- Title       : TutorVHDL
-- Design      : TutorVHDL
-- Author      : J.Kasperek & P.J.Rajda
-- Company     : AGH Krakw
--
-------------------------------------------------------------------------------
--
-- Description : 
-- Synchronous 3bits binary up counter with synchronous reset
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_unsigned.all;


entity TutorVHDL is
	port(
		CLR : in STD_LOGIC;			-- clear signal 
		CLK : in STD_LOGIC;			-- clock 
		Q : out STD_LOGIC_VECTOR(2 downto 0)	-- outputs
		);
end TutorVHDL;


architecture TutorVHDL_var of TutorVHDL is	 
	-- internal signal Q_INT
	signal Q_INT: STD_LOGIC_VECTOR(2 downto 0);
begin 
	process (CLK)
		-- internal variable Q_INT
		variable Q_INT: STD_LOGIC_VECTOR(2 downto 0);
	begin
		if CLK'event and CLK='1' then
			if CLR='1' then
				Q_INT := "000";
			else	
				Q_INT := Q_INT+1;
			end if;
		end if;	
		Q <= Q_INT;
	end process;
end TutorVHDL_var;



architecture TutorVHDL_sig_bad_0 of TutorVHDL is	 
	-- internal signal Q_INT
	signal Q_INT: STD_LOGIC_VECTOR(2 downto 0);	-- variable changed to signal
begin 
	process (CLK)
	begin
		if CLK'event and CLK = '1' then
			if CLR = '1' then
				Q_INT <= "000";					-- variable changed to signal
			else
				Q_INT <= Q_INT+1;				-- variable changed to signal
			end if;
		end if;	
		Q <= Q_INT;
	end process;
end TutorVHDL_sig_bad_0;



architecture TutorVHDL_sig_bad_1 of TutorVHDL is	 
	-- internal signal Q_INT
	signal Q_INT: STD_LOGIC_VECTOR(2 downto 0);
begin 
	process (CLK)
	begin
		if CLK'event and CLK = '0' then	-- changing to falling CLK edge detection
			if CLR = '1' then
				Q_INT <= "000";
			else
				Q_INT <= Q_INT+1;
			end if;
		end if;	
		Q <= Q_INT;
	end process;
end TutorVHDL_sig_bad_1;



architecture TutorVHDL_sig_bad_2 of TutorVHDL is	 
	-- internal signal Q_INT
	signal Q_INT: STD_LOGIC_VECTOR(2 downto 0);
begin 
	process (CLK)
	begin
		if CLK'event and CLK = '1' then
			if CLR = '1' then
				Q_INT <= "000";
			else	
				Q_INT <= Q_INT+1;
			end if;
			Q <= Q_INT;					-- Q asignment moved into CLK edge detection condition
		end if;	
	end process;
end TutorVHDL_sig_bad_2;



architecture TutorVHDL_sig_better_3 of TutorVHDL is	 
	-- internal signal Q_INT
	signal Q_INT: STD_LOGIC_VECTOR(2 downto 0);
begin 
	process (CLK, Q_INT)				 -- extended sensitivity list
	begin
		if CLK'event and CLK = '1' then
			if CLR = '1' then
				Q_INT <= "000";
			else	
				Q_INT <= Q_INT+1;
			end if;
		end if;	
		Q <= Q_INT;
	end process;
end TutorVHDL_sig_better_3;



architecture TutorVHDL_sig_better_4 of TutorVHDL is	 
	-- internal signal Q_INT
	signal Q_INT: STD_LOGIC_VECTOR(2 downto 0);
begin 
	process (CLK)
	begin
		if CLK'event and CLK = '1' then
			if CLR = '1' then
				Q_INT <= "000";
				Q <= "000";					-- mirror behaviour, but synthesable
			else	
				Q_INT <= Q_INT+1;
				Q <= Q_INT+1;				-- mirror behaviour, but synthesable
			end if;
		end if;	
	end process;
end TutorVHDL_sig_better_4;



architecture TutorVHDL_sig_good of TutorVHDL is	 
	-- internal signal Q_INT
	signal Q_INT: STD_LOGIC_VECTOR(2 downto 0);
begin 
	process (CLK)
	begin
		if CLK'event and CLK = '1' then
			if CLR = '1' then
				Q_INT <= "000";
			else	
				Q_INT <= Q_INT+1;
			end if;
		end if;	
	end process;
	Q <= Q_INT;							-- Q assignment chandged from procedural to concurrent
end TutorVHDL_sig_good;

